local DynamicFighter = require "game.DynamicFighter"

local Actor = DynamicFighter:extend()

Actor.MOTION_APPEAR_1 = {{code = "PerformAnimation", args = {"Jump"}}, {code = "JumpBy", args = {160, 480, 32, 16}}, {code = "JumpBack", args = {32, 16}}, {code = "PerformAnimation", args = {"Crouch"}}, {code = "Wait", args = {4}}, {code = "RefreshAnimation"}, {code = "End"}}
Actor.MOTION_APPEAR_2 = {{code = "PerformAnimation", args = {"Jump"}}, {code = "JumpBy", args = {-256, 0, 32, 16}}, {code = "JumpBack", args = {32, 16}}, {code = "PerformAnimation", args = {"Crouch"}}, {code = "Wait", args = {4}}, {code = "RefreshAnimation"}, {code = "End"}}
Actor.MOTION_APPEAR_3 = {}
Actor.MOTION_APPEAR_4 = {}
Actor.MOTION_ATTACK = {{code = "PerformAnimation", args = {"Crouch"}}, {code = "Wait", args = {4}}, {code = "PerformAnimation", args = {"Jump"}}, {code = "JumpBy", args = {-32, 0, 32, 16}}, {code = "PerformAnimation", args = {"Attack"}}, {code = "End"}}
Actor.MOTION_CAST = {{code = "PerformAnimation", args = {"Crouch"}}, {code = "Wait", args = {4}}, {code = "PerformAnimation", args = {"Jump"}}, {code = "JumpBy", args = {-32, 0, 32, 16}}, {code = "PerformAnimation", args = {"Cast"}}, {code = "End"}}
Actor.MOTION_ITEM = {{code = "PerformAnimation", args = {"Crouch"}}, {code = "Wait", args = {4}}, {code = "PerformAnimation", args = {"Jump"}}, {code = "JumpBy", args = {-32, 0, 32, 16}}, {code = "PerformAnimation", args = {"Item"}}, {code = "End"}}
Actor.MOTION_RETURN = {{code = "PerformAnimation", args = {"Crouch"}}, {code = "Wait", args = {4}}, {code = "PerformAnimation", args = {"Jump"}}, {code = "JumpBack", args = {32, 16}}, {code = "PerformAnimation", args = {"Crouch"}}, {code = "Wait", args = {4}}, {code = "RefreshAnimation"}, {code = "End"}}
Actor.MOTION_DAMAGE = {{code = "PerformAnimation", args = {"Damage"}}, {code = "Wait", args = {16}}, {code = "RefreshAnimation"}, {code = "End"}}
Actor.MOTION_MISS = {{code = "PerformAnimation", args = {"Miss"}}, {code = "Wait", args = {16}}, {code = "RefreshAnimation"}, {code = "End"}}
Actor.MOTION_ESCAPE = {{code = "PerformAnimation", args = {"Escape"}}, {code = "MoveBy", args = {320, 0, 48}}, {code = "End"}}
Actor.MOTION_WIN = {{code = "PerformAnimation", args = {"Crouch"}}, {code = "Wait", args = {4}}, {code = "PerformAnimation", args = {"Win1"}}, {code = "Jump", args = {32, 32}}, {code = "PerformAnimation", args = {"Crouch"}}, {code = "Wait", args = {4}}, {code = "PerformAnimation", args = {"Win2"}}, {code = "End"}}

Actor.ANIMATION_IDLE = {1, 3}
Actor.ANIMATION_CRITICAL = {2, 3}
Actor.ANIMATION_DEFEAT = {3, 3}
Actor.ANIMATION_DEFENSE = {4, 3}
Actor.ANIMATION_PREPARE = {5, 3}
Actor.ANIMATION_DAMAGE = {6, 5, false}
Actor.ANIMATION_MISS = {7, 5, false}
Actor.ANIMATION_CROUCH = {8, 3}
Actor.ANIMATION_JUMP = {9, 5}
Actor.ANIMATION_ATTACK = {10, 6, false}
Actor.ANIMATION_CAST = {11, 4}
Actor.ANIMATION_ITEM = {12, 4, false}
Actor.ANIMATION_ESCAPE = {13, 5}
Actor.ANIMATION_WIN_1 = {14, 4, false}
Actor.ANIMATION_WIN_2 = {15, 4, false}

function Actor:initialize(id)
	DynamicFighter.initialize(self)
	-- Shadow image
	self.shadowImage = RM:getSystemImage("shadow.png")
	-- Animation
	self.animationLoop = true
	self.animationIndex = 1
	self.animationFrame = 1
	self.animationTimer = 0
	self.animationSpeed = 3
	-- Id/stats
	self.id = id
	self.level = 1
end

function Actor:isActor()
	return true
end

function Actor:isSkillUsable(id)
	return self:getMp() >= dataSkills[id].mpCost and not self:isSkillLocked(dataSkills[id].type)
end

function Actor:hasTechniques()
	-- TODO
	return false
end

function Actor:hasMagic()
	-- TODO
	return true
end

function Actor:hasSummon()
	-- TODO
	return false
end

function Actor:getShadowQuadW()
	return self.shadowImage:getWidth()
end

function Actor:getShadowQuadH()
	return self.shadowImage:getHeight()
end

function Actor:getQuadIndex()
	return (self.animationIndex - 1) * 4 + self.animationFrame
end

function Actor:getQuadW()
	return (self.image and self.image:getWidth() / 16) or 0
end

function Actor:getQuadH()
	return (self.image and self.image:getHeight() / 4) or 0
end

function Actor:getW()
	return self:getQuadW() / 2
end

function Actor:getId()
	return self.id
end

function Actor:getTechniques()
	-- TODO
	local r = {}
	for i, v in ipairs(dataSkills) do
		if v.type == 2 then
			table.insert(r, i)
		end
	end
	return r
end

function Actor:getMagic()
	-- TODO
	local r = {}
	for i, v in ipairs(dataSkills) do
		if v.type == 3 then
			table.insert(r, i)
		end
	end
	return r
end

function Actor:getSummon()
	local r = {}
	for i, v in ipairs(dataSkills) do
		if v.type == 4 then
			table.insert(r, i)
		end
	end
	return r
end

function Actor:getName()
	return dataActors[self.id].name
end

function Actor:getFace()
	return dataActors[self.id].face
end

function Actor:getAttackAnimation()
	return dataActors[self.id].attackAnimation
end

function Actor:getLevel()
	return self.level
end

function Actor:setAnimation(index, speed, loop)
	if index ~= self.animationIndex then
		self.animationFrame = 1
		self.animationTimer = 0
	end
	self.animationLoop = loop == nil or loop
	self.animationIndex = index
	self.animationSpeed = speed
end

function Actor:setupNewGame()
	local data = dataActors[self.id]
	-- Image
	self.image = RM:getActorImage(data.actorImage)
	self.quads = {}
	local qw = self:getQuadW()
	local qh = self:getQuadH()
	for i = 0, 3 do
		for y = 0, 3 do
			for x = 0, 3 do
				local qx = i * qw * 4 + x * qw
				local qy = y * qh
				table.insert(self.quads, love.graphics.newQuad(qx, qy, qw, qh, qw * 16, qh * 4))
			end
		end
	end
	-- Stats
	self.level = data.level
	self.mhp = data.mhp[self.level]
	self.mmp = data.mmp[self.level]
	self.atk = data.atk[self.level]
	self.def = data.def[self.level]
	self.mat = data.mat[self.level]
	self.mdf = data.mdf[self.level]
	self.spd = data.spd[self.level]
	self.crt = data.crt
	self.hit = data.hit
	self.eva = data.eva
	self.hp = self.mhp
	self.mp = self.mmp
	-- Statuses/buffs
	self:clearStatuses()
	self:clearBuffs()
	-- Refresh
	self:refresh()
end

function Actor:refreshMotion()
	if self:isDead() then
		self:performMotion("Defeat")
	else
		self:refreshAnimation()
	end
end

function Actor:performMotion(motion, ...)
	if motion == "Appear" then
		if self:isConscious() then
			local id = dataActors[self.id].appearMotion
			-- Jump (TL)
			if id == 1 then
				self:moveBy(-320, -480, 0)
				self:setMotion(Actor.MOTION_APPEAR_1)
			end
			-- Jump (R)
			if id == 2 then
				self:moveBy(320, 0, 0)
				self:setMotion(Actor.MOTION_APPEAR_2)
			end
			-- TODO
		else
			self:refreshMotion()
		end
	end
	if motion == "Defeat" then
		self:performAnimation("Defeat")
	end
	if motion == "Damage" then
		self:setMotion(Actor.MOTION_DAMAGE)
	end
	if motion == "Miss" then
		self:setMotion(Actor.MOTION_MISS)
	end
	if motion == "Attack" then
		self:setMotion(Actor.MOTION_ATTACK)
	end
	if motion == "Custom" then
		-- TODO
		self:setMotion(Actor.MOTION_ATTACK)
		-- TODO
	end
	if motion == "Cast" then
		self:setMotion(Actor.MOTION_CAST)
	end
	if motion == "Item" then
		self:setMotion(Actor.MOTION_ITEM)
	end
	if motion == "Return" then
		self:setMotion(Actor.MOTION_RETURN)
	end
	if motion == "Escape" then
		if self:isConscious() then
			self:setMotion(Actor.MOTION_ESCAPE)
		end
	end
	if motion == "Win" then
		if self:isConscious() then
			self:setMotion(Actor.MOTION_WIN)
		end
	end
end

function Actor:refreshAnimation()
	if self:isDefending() then
		self:performAnimation("Defense")
	elseif self:isPreparing() then
		self:performAnimation("Prepare")
	elseif self:isHpCritical() then
		self:performAnimation("Critical")
	else
		self:performAnimation("Idle")
	end
end

function Actor:performAnimation(animation)
	if animation == "Idle" then
		self:setAnimation(unpack(Actor.ANIMATION_IDLE))
	end
	if animation == "Critical" then
		self:setAnimation(unpack(Actor.ANIMATION_CRITICAL))
	end
	if animation == "Defeat" then
		self:setAnimation(unpack(Actor.ANIMATION_DEFEAT))
	end
	if animation == "Defense" then
		self:setAnimation(unpack(Actor.ANIMATION_DEFENSE))
	end
	if animation == "Prepare" then
		self:setAnimation(unpack(Actor.ANIMATION_PREPARE))
	end
	if animation == "Damage" then
		self:setAnimation(unpack(Actor.ANIMATION_DAMAGE))
	end
	if animation == "Miss" then
		self:setAnimation(unpack(Actor.ANIMATION_MISS))
	end
	if animation == "Crouch" then
		self:setAnimation(unpack(Actor.ANIMATION_CROUCH))
	end
	if animation == "Jump" then
		self:setAnimation(unpack(Actor.ANIMATION_JUMP))
	end
	if animation == "Attack" then
		self:setAnimation(unpack(Actor.ANIMATION_ATTACK))
	end
	if animation == "Cast" then
		self:setAnimation(unpack(Actor.ANIMATION_CAST))
	end
	if animation == "Item" then
		self:setAnimation(unpack(Actor.ANIMATION_ITEM))
	end
	if animation == "Escape" then
		self:setAnimation(unpack(Actor.ANIMATION_ESCAPE))
	end
	if animation == "Win1" then
		self:setAnimation(unpack(Actor.ANIMATION_WIN_1))
	end
	if animation == "Win2" then
		self:setAnimation(unpack(Actor.ANIMATION_WIN_2))
	end
end

function Actor:update()
	DynamicFighter.update(self)
	self.animationTimer = self.animationTimer + 1
	if self.animationTimer > 128 / (2 ^ self.animationSpeed) then
		if self.animationFrame < 4 or self.animationLoop then
			self.animationFrame = self.animationFrame % 4 + 1
		end
		self.animationTimer = 0
	end
end

function Actor:draw()
	-- Shadow
	local x = self:getX()
	local y = self:getY()
	local ox = self:getShadowQuadW() / 2
	local oy = self:getShadowQuadH()
	self.shader:send("opacity", self.a)
	self.shader:sendColor("color", {self:getColor()})
	self.shader:sendColor("flash", {self:getFlash()})
	self.shader:sendColor("displayColor", {gameDisplay:getColor()})
	self.shader:sendColor("displayFlash", {gameDisplay:getFlash()})
	love.graphics.setShader(self.shader)
	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(self.shadowImage, x, y, 0, 1, 1, ox, oy)
	-- Entity
	DynamicFighter.draw(self)
end

return Actor