local Group = require "game.Group"
local BattleState = require "state.BattleState"

local ActorGroup = Group:extend()

function ActorGroup:initialize()
	Group.initialize(self)
	self.items = {}
	self.itemLists = {}
	self.itemSublists = {}
end

function ActorGroup:isInBattle()
	return SM:peekState():isA(BattleState)
end

function ActorGroup:getMaxBattleSize()
	return 4
end

function ActorGroup:getItems()
	return self.itemLists[1] or {}
end

function ActorGroup:getEquipments()
	return self.itemLists[2] or {}
end

function ActorGroup:getKeyItems()
	return self.itemLists[3] or {}
end

function ActorGroup:getWeapons()
	return self.itemSublists[2][1] or {}
end

function ActorGroup:getShields()
	return self.itemSublists[2][2] or {}
end

function ActorGroup:getArmors()
	return self.itemSublists[2][3] or {}
end

function ActorGroup:getAccessories()
	return self.itemSublists[2][4] or {}
end

function ActorGroup:getRelics()
	return self.itemSublists[2][5] or {}
end

function ActorGroup:getItemAmount(id)
	return self.items[id] or 0
end

function ActorGroup:changeItemAmount(id, value)
	local oa = self:getItemAmount(id)
	local na = math.max(0, math.min(99, oa + value))
	if oa == 0 and na > 0 then
		local t = dataItems[id].type
		local s = dataItems[id].subtype
		self.itemLists[t] = self.itemLists[t] or {}
		table.insert(self.itemLists[t], id)
		table.sort(self.itemLists[t])
		if s > 0 then
			self.itemSublists[t][s] = self.itemSublists[t][s] or {}
			table.insert(self.itemSublists[t][s], id)
			table.sort(self.itemSublists[t][s])
		end
	end
	if na == 0 and oa > 0 then
		local t = dataItems[id].type
		local s = dataItems[id].subtype
		for i = #self.itemLists[t], 1, -1 do
			if self.itemLists[t][i] == id then
				table.remove(self.itemLists[t], i)
				break
			end
		end
		if s > 0 then
			for i = #self.itemSublists[t][s], 1, -1 do
				if self.itemSublists[t][s][i] == id then
					table.remove(self.itemSublists[t][s], i)
					break
				end
			end
		end
	end
	self.items[id] = na
	-- TODO refresh eventi
end

function ActorGroup:setupNewGame()
	-- Members
	self:clear()
	-- Items
	self.items = {}
	self.itemLists = {}
	self.itemSublists = {}
	-- TODO
	for _, v in ipairs(gameActors) do
		gameActorGroup:addMember(v)
	end
	for i, _ in ipairs(dataItems) do
		self:changeItemAmount(i, 2)
	end
	-- TODO
end

function ActorGroup:setupBattlePosition()
	for i, v in ipairs(self:getBattleMembers()) do
		local x = 512 + (i + self:getMaxBattleSize() - self:getBattleSize() - 1) * 16
		local y = 160 + (i - 1) * 48
		v:setPosition(x, y)
	end
end

return ActorGroup