local Object = require "core.Object"

local Display = Object:extend()

function Display:initialize()
    self.r = 0.5
    self.g = 0.5
    self.b = 0.5
    self.s = 0.5
    self.tr = 0.5
    self.tg = 0.5
    self.tb = 0.5
    self.ts = 0.5
    self.sr = 0
    self.sg = 0
    self.sb = 0
    self.ss = 0
    self.fr = 1
    self.fg = 1
    self.fb = 1
    self.fi = 0
    self.fs = 0
end

function Display:isBusy()
    return self:isColoring() or self:isFlashing()
end

function Display:isColoring()
    return self.r ~= self.tr or self.g ~= self.tg or self.b ~= self.tb or self.s ~= self.ts
end

function Display:isFlashing()
    return self.fi > 0
end

function Display:getColor()
    return self.r, self.g, self.b, self.s
end

function Display:getFlash()
    return self.fr, self.fg, self.fb, self.fi
end

function Display:color(r, g, b, s, duration)
    self.tr = r
    self.tg = g
    self.tb = b
    self.ts = s
    if (duration or 0) > 0 then
        self.sr = math.abs(r - self.r) / duration
        self.sg = math.abs(g - self.g) / duration
        self.sb = math.abs(b - self.b) / duration
        self.ss = math.abs(s - self.s) / duration
    else
        self.r = r
        self.g = g
        self.b = b
        self.s = s
        self.sr = 0
        self.sg = 0
        self.sb = 0
        self.ss = 0
    end
end

function Display:flash(r, g, b, intensity, duration)
    self.fr = r
	self.fg = g
	self.fb = b
	self.fi = intensity
	self.fs = intensity / duration
end

function Display:update()
    if self.r < self.tr then
        self.r = math.min(self.tr, self.r + self.sr)
    end
    if self.r > self.tr then
        self.r = math.max(self.tr, self.r - self.sr)
    end
    if self.g < self.tg then
        self.g = math.min(self.tg, self.g + self.sg)
    end
    if self.g > self.tg then
        self.g = math.max(self.tg, self.g - self.sg)
    end
    if self.b < self.tb then
        self.b = math.min(self.tb, self.b + self.sb)
    end
    if self.b > self.tb then
        self.b = math.max(self.tb, self.b - self.sb)
    end
    if self.s < self.ts then
        self.s = math.min(self.ts, self.s + self.ss)
    end
    if self.s > self.ts then
        self.s = math.max(self.ts, self.s - self.ss)
    end
    if self.fi > 0 then
        self.fi = math.max(0, self.fi - self.fs)
    end
end

return Display