local Fighter = require "game.Fighter"

local DynamicFighter = Fighter:extend()

function DynamicFighter:initialize()
	Fighter.initialize(self)
	-- Motion
	self.motion = {}
	self.motionIndex = 1
	self.motionTimer = 0
	-- Position
	self.x = 0
	self.y = 0
	self.ox = 0
	self.oy = 0
	self.mx = 0
	self.my = 0
	self.msx = 0
	self.msy = 0
	-- Jump
	self.jh = 0
	self.jd = 0
	self.jc = 0
end

function DynamicFighter:isMotionBusy()
	return self.motionIndex <= #self.motion
end

function DynamicFighter:isMoving()
	return self.x ~= self.mx or self.y ~= self.my or self.jc < self.jd
end

function DynamicFighter:getX()
	return self.x
end

function DynamicFighter:getY()
	return self.y
end

function DynamicFighter:getZ()
	return DynamicFighter.PRIORITY_FIGHTER + self:getY()
end

function DynamicFighter:getYOffset()
	if self.jc < self.jd then
		return self.jc * (1 / self.jd - self.jc / (self.jd ^ 2)) * self.jh * 4
	end
	return 0
end

function DynamicFighter:setMotion(motion)
	self.motion = motion
	self.motionIndex = 1
	self.motionTimer = 0
end

function DynamicFighter:setPosition(x, y)
	self.x = x
	self.y = y
	self.ox = x
	self.oy = y
	self.mx = x
	self.my = y
	self.msx = 0
	self.msy = 0
end

function DynamicFighter:playSound(name)
	AM:playSound(name)
end

function DynamicFighter:wait(duration)
	if self.motionTimer > 0 then
		self.motionTimer = self.motionTimer - 1
	else
		self.motionTimer = duration
	end
	if self.motionTimer > 0 then
		self.motionIndex = self.motionIndex - 1
	end
end

function DynamicFighter:moveTo(x, y, duration)
	self.mx = x
	self.my = y
	if (duration or 0) > 0 then
		self.msx = math.abs(x - self.x) / duration
		self.msy = math.abs(y - self.y) / duration
	else
		self.x = x
		self.y = y
		self.msx = 0
		self.msy = 0
	end
end

function DynamicFighter:moveBy(x, y, duration)
	self:moveTo(self.x + x, self.y + y, duration)
end

function DynamicFighter:moveBack(duration)
	self:moveTo(self.ox, self.oy, duration)
end

function DynamicFighter:jump(height, duration)
	self.jh = height
	self.jd = duration
	self.jc = 0
end

function DynamicFighter:jumpTo(x, y, height, duration)
	self:jump(height, duration)
	self:moveTo(x, y, duration)
end

function DynamicFighter:jumpBy(x, y, height, duration)
	self:jumpTo(self.x + x, self.y + y, height, duration)
end

function DynamicFighter:jumpBack(height, duration)
	self:jumpTo(self.ox, self.oy, height, duration)
end

function DynamicFighter:refreshMotion()
end

function DynamicFighter:performMotion(motion, ...)
end

function DynamicFighter:updateMotion(elem)
	local code = elem.code
	local args = elem.args or {}
	local name = code:sub(1, 1):lower() .. code:sub(2)
	if self[name] then
		self[name](self, unpack(args))
	end
end

function DynamicFighter:update()
	Fighter.update(self)
	-- Motion
	if not self:isMoving() then
		if self.motionIndex <= #self.motion then
			self:updateMotion(self.motion[self.motionIndex])
			self.motionIndex = self.motionIndex + 1
		end
	end
	-- Movement
	if self.jc < self.jd then
		self.jc = self.jc + 1
	end
	if self.y > self.my then
		self.y = math.max(self.my, self.y - self.msy)
	end
	if self.x < self.mx then
		self.x = math.min(self.mx, self.x + self.msx)
	end
	if self.y < self.my then
		self.y = math.min(self.my, self.y + self.msy)
	end
	if self.x > self.mx then
		self.x = math.max(self.mx, self.x - self.msx)
	end
end

return DynamicFighter