local DynamicFighter = require "game.DynamicFighter"
local Action = require "game.Action"

local Enemy = DynamicFighter:extend()

Enemy.MOTION_APPEAR_1 = {{code = "JumpBack", args = {16, 16}}, {code = "End", args = {}}}
Enemy.MOTION_DEFEAT_1 = {{code = "PlaySound", args = {"collapse01.wav"}}, {code = "Color", args = {0.75, 0.45, 0.75, 0.5, 0}}, {code = "Fade", args = {32}}, {code = "Wait", args = {32}}, {code = "End"}}
Enemy.MOTION_ACT = {{code = "Flash", args = {1, 1, 0.3, 0.6, 8}}, {code = "Wait", args = {8}}, {code = "Flash", args = {1, 1, 0.3, 0.6, 8}}, {code = "Wait", args = {8}}, {code = "End"}}

function Enemy:initialize(id)
	DynamicFighter.initialize(self)
	local data = dataEnemies[id]
	-- Image
	self.image = RM:getEnemyImage(data.image)
	-- Id
	self.id = id
	-- Stats
	self.mhp = data.mhp
	self.mmp = data.mmp
	self.atk = data.atk
	self.def = data.def
	self.mat = data.mat
	self.mdf = data.mdf
	self.spd = data.spd
	self.crt = data.crt
	self.hit = data.hit
	self.eva = data.eva
	self.hp = self.mhp
	self.mp = self.mmp
	-- Status/debuff/attribute rates
	self.statusRate = {unpack(data.statusRate)}
	self.debuffRate = {unpack(data.debuffRate)}
	self.attributeRate = {unpack(data.attributeRate)}
end

function Enemy:isEnemy()
	return true
end

function Enemy:getId()
	return self.id
end

function Enemy:getName()
	return dataEnemies[self.id].name
end

function Enemy:getAttackAnimation()
	return dataEnemies[self.id].attackAnimation
end

function Enemy:getStatusRate(id)
	local r = DynamicFighter.getStatusRate(self, id)
	for _, v in ipairs(self.statusRate) do
		if v.id == id then
			r = r * v.rate
		end
	end
	return r
end

function Enemy:getDebuffRate(id)
	local r = DynamicFighter.getDebuffRate(self, id)
	for _, v in ipairs(self.debuffRate) do
		if v.id == id then
			r = r * v.rate
		end
	end
	return r
end

function Enemy:getAttributeRate(id)
	local r = DynamicFighter.getAttributeRate(self, id)
	for _, v in ipairs(self.attributeRate) do
		if v.id == id then
			r = r * v.rate
		end
	end
	return r
end

function Enemy:setupAutoAction()
	-- TODO
	self:setAction(Action:new("Skill", Action.SKILL_ATTACK_ID, self, false, "A", 0))
end

function Enemy:refreshMotion()
	if self:isDead() then
		self:performMotion("Defeat")
	end
end

function Enemy:performMotion(motion, ...)
	if motion == "Appear" then
		if self:isConscious() then
			local id = dataEnemies[self.id].appearMotion
			-- Jump (L)
			if id == 1 then
				self:moveBy(-480, 0, 0)
				self:setMotion(Enemy.MOTION_APPEAR_1)
			end
			-- TODO
		else
			self:refreshMotion()
		end
	end
	if motion == "Defeat" then
		local id = dataEnemies[self.id].defeatMotion
		-- Normal
		if id == 1 then
			self:setMotion(Enemy.MOTION_DEFEAT_1)
		end
		-- TODO
	end
	if motion == "Attack" or motion == "Custom" or motion == "Cast" or motion == "Item" then
		self:setMotion(Enemy.MOTION_ACT)
	end
end

return Enemy