local Entity = require "game.Entity"
local Action = require "game.Action"

local Fighter = Entity:extend()

Fighter.STATUS_DEATH_ID = 1

Fighter.STAT_MHP_ID = 1
Fighter.STAT_MMP_ID = 2
Fighter.STAT_ATK_ID = 3
Fighter.STAT_DEF_ID = 4
Fighter.STAT_MAT_ID = 5
Fighter.STAT_MDF_ID = 6
Fighter.STAT_SPD_ID = 7
Fighter.STAT_CRT_ID = 8
Fighter.STAT_HIT_ID = 9
Fighter.STAT_EVA_ID = 10

function Fighter:initialize()
	Entity.initialize(self)
	-- Statuses
	self.statuses = {}
	self.statusCount = {}
	-- Buffs
	self.buffs = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
	self.buffCount = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
	-- Action
	self.action = nil
	-- Stats
	self.mhp = 1
	self.hp = 1
	self.mmp = 0
	self.mp = 1
	self.atk = 0
	self.def = 0
	self.mat = 0
	self.mdf = 0
	self.spd = 0
	self.crt = 0
	self.hit = 1
	self.eva = 0
end

function Fighter:isActor()
	return false
end

function Fighter:isEnemy()
	return false
end

function Fighter:isDead()
	return self:hasStatus(Fighter.STATUS_DEATH_ID)
end

function Fighter:isRestricted()
	return self:getRestriction() > 0
end

function Fighter:isConscious()
	return self:getRestriction() < 4
end

function Fighter:isSkillLocked(id)
	for _, v1 in ipairs(self.statuses) do
		for _, v2 in ipairs(dataStatus[v1].lockedSkills) do
			if v2 == id then
				return true
			end
		end
	end
	return false
end

function Fighter:isDamageInverted()
	for _, v in ipairs(self.statuses) do
		if dataStatus[v].invertDamage then
			return true
		end
	end
	return false
end

function Fighter:isHpCritical()
	return self.hp < self:getMhp() / 4
end

function Fighter:isMpCritical()
	return self.mp < self:getMmp() / 4
end

function Fighter:isDefending()
	-- TODO
	return false
end

function Fighter:isPreparing()
	-- TODO
	return false
end

function Fighter:hasStatus(id)
	for _, v in ipairs(self.statuses) do
		if v == id then
			return true
		end
	end
	return false
end

function Fighter:hasBuff(id)
	return self.buffs[id] > 0
end

function Fighter:hasDebuff(id)
	return self.buffs[id] < 0
end

function Fighter:getRestriction()
	local r = 0
	for _, v in ipairs(self.statuses) do
		r = math.max(r, dataStatus[v].restriction)
	end
	return r
end

function Fighter:getAction()
	return self.action
end

function Fighter:getName()
	return ""
end

function Fighter:getAttackAnimation()
	return 0
end

function Fighter:getBaseMhp()
	return self.mhp
end

function Fighter:getBaseMmp()
	return self.mmp
end

function Fighter:getBaseAtk()
	return self.atk
end

function Fighter:getBaseDef()
	return self.def
end

function Fighter:getBaseMat()
	return self.mat
end

function Fighter:getBaseMdf()
	return self.mdf
end

function Fighter:getBaseSpd()
	return self.spd
end

function Fighter:getBaseCrt()
	return self.crt
end

function Fighter:getBaseHit()
	return self.hit
end

function Fighter:getBaseEva()
	return self.eva
end

function Fighter:getPlusMhp()
	return 0
end

function Fighter:getPlusMmp()
	return 0
end

function Fighter:getPlusAtk()
	return 0
end

function Fighter:getPlusDef()
	return 0
end

function Fighter:getPlusMat()
	return 0
end

function Fighter:getPlusMdf()
	return 0
end

function Fighter:getPlusSpd()
	return 0
end

function Fighter:getPlusCrt()
	return 0
end

function Fighter:getPlusHit()
	return 0
end

function Fighter:getPlusEva()
	return 0
end

function Fighter:getMhpRate()
	return self.buffs[Fighter.STAT_MHP_ID] * 0.25 + 1
end

function Fighter:getMmpRate()
	return self.buffs[Fighter.STAT_MMP_ID] * 0.25 + 1
end

function Fighter:getAtkRate()
	return self.buffs[Fighter.STAT_ATK_ID] * 0.25 + 1
end

function Fighter:getDefRate()
	return self.buffs[Fighter.STAT_DEF_ID] * 0.25 + 1
end

function Fighter:getMatRate()
	return self.buffs[Fighter.STAT_MAT_ID] * 0.25 + 1
end

function Fighter:getMdfRate()
	return self.buffs[Fighter.STAT_MDF_ID] * 0.25 + 1
end

function Fighter:getSpdRate()
	return self.buffs[Fighter.STAT_SPD_ID] * 0.25 + 1
end

function Fighter:getCrtRate()
	return self.buffs[Fighter.STAT_CRT_ID] * 0.25 + 1
end

function Fighter:getHitRate()
	return self.buffs[Fighter.STAT_HIT_ID] * 0.25 + 1
end

function Fighter:getEvaRate()
	return self.buffs[Fighter.STAT_EVA_ID] * 0.25 + 1
end

function Fighter:getMhp()
	return math.max(1, math.min((self:isActor() and 999) or math.huge, math.floor((self:getBaseMhp() + self:getPlusMhp()) * self:getMhpRate())))
end

function Fighter:getMmp()
	return math.max(0, math.min((self:isActor() and 999) or math.huge, math.floor((self:getBaseMmp() + self:getPlusMmp()) * self:getMmpRate())))
end

function Fighter:getAtk()
	return math.max(0, math.min(999, math.floor((self:getBaseAtk() + self:getPlusAtk()) * self:getAtkRate())))
end

function Fighter:getDef()
	return math.max(0, math.min(999, math.floor((self:getBaseDef() + self:getPlusDef()) * self:getDefRate())))
end

function Fighter:getMat()
	return math.max(0, math.min(999, math.floor((self:getBaseMat() + self:getPlusMat()) * self:getMatRate())))
end

function Fighter:getMdf()
	return math.max(0, math.min(999, math.floor((self:getBaseMdf() + self:getPlusMdf()) * self:getMdfRate())))
end

function Fighter:getSpd()
	return math.max(0, math.min(999, math.floor((self:getBaseSpd() + self:getPlusSpd()) * self:getSpdRate())))
end

function Fighter:getCrt()
	return math.max(0, math.min(1, (self:getBaseCrt() + self:getPlusCrt()) * self:getCrtRate()))
end

function Fighter:getHit()
	return math.max(0, math.min(1, (self:getBaseHit() + self:getPlusHit()) * self:getHitRate()))
end

function Fighter:getEva()
	return math.max(0, math.min(1, (self:getBaseEva() + self:getPlusEva()) * self:getEvaRate()))
end

function Fighter:getHp()
	return self.hp
end

function Fighter:getMp()
	return self.mp
end

function Fighter:getDamageRate()
	local r = 1
	for _, v in ipairs(self.statuses) do
		r = r * dataStatus[v].damageRate
	end
	return r
end

function Fighter:getHpRecoveryRate()
	local r = 0
	for _, v in ipairs(self.statuses) do
		r = r + dataStatus[v].hpRecoveryRate
	end
	return r
end

function Fighter:getMpRecoveryRate()
	local r = 0
	for _, v in ipairs(self.statuses) do
		r = r + dataStatus[v].mpRecoveryRate
	end
	return r
end

function Fighter:getCounterattackRate()
	return 0
end

function Fighter:getStatusRate(id)
	local r = 1
	for _, v1 in ipairs(self.statuses) do
		for _, v2 in ipairs(dataStatus[v1].statusRate) do
			if v2.id == id then
				r = r * v2.rate
			end
		end
	end
	return r
end

function Fighter:getDebuffRate(id)
	local r = 1
	for _, v1 in ipairs(self.statuses) do
		for _, v2 in ipairs(dataStatus[v1].debuffRate) do
			if v2.id == id then
				r = r * v2.rate
			end
		end
	end
	return r
end

function Fighter:getAttributeRate(id)
	local r = 1
	for _, v1 in ipairs(self.statuses) do
		for _, v2 in ipairs(dataStatus[v1].attributeRate) do
			if v2.id == id then
				r = r * v2.rate
			end
		end
	end
	return r
end

function Fighter:getPhysicalAttributes()
	return {}
end

function Fighter:getPhysicalStatuses()
	return {}
end

function Fighter:setAction(action)
	self.action = action
end

function Fighter:setHp(hp)
	self.hp = hp
	self:refresh()
end

function Fighter:setMp(mp)
	self.mp = mp
	self:refresh()
end

function Fighter:changeHp(value)
	self:setHp(self.hp + value)
end

function Fighter:changeMp(value)
	self:setMp(self.mp + value)
end

function Fighter:addStatus(id)
	if not self:hasStatus(id) then
		if id == Fighter.STATUS_DEATH_ID then
			self.hp = 0
			self:clearStatuses()
			self:clearBuffs()
		end
		table.insert(self.statuses, id)
		self:refresh()
	end
	if not self.statusCount[id] or self.statusCount[id] > 0 then
		local min = dataStatus[id].removeMinCount
		local max = dataStatus[id].removeMaxCount
		self.statusCount[id] = math.max(self.statusCount[id] or 0, math.random(min, max))
	end
end

function Fighter:addBuff(id, turns)
	self.buffs[id] = math.min(2, self.buffs[id] + 1)
	if self:hasBuff(id) then
		self.buffCount[id] = turns
	end
	self:refresh()
end

function Fighter:addDebuff(id, turns)
	self.buffs[id] = math.max(-2, self.buffs[id] - 1)
	if self:hasDebuff(id) then
		self.buffCount[id] = turns
	end
	self:refresh()
end

function Fighter:removeStatus(id)
	if self:hasStatus(id) then
		if id == Fighter.STATUS_DEATH_ID then
			if self.hp == 0 then
				self.hp = 1
			end
		end
		for i = #self.statuses, 1, -1 do
			if self.statuses[i] == id then
				table.remove(self.statuses, i)
			end
		end
		self:refresh()
	end
	self.statusCount[id] = nil
end

function Fighter:removeBuff(id)
	if self:hasBuff(id) then
		self.buffs[id] = 0
		self.buffCount[id] = 0
		self:refresh()
	end
end

function Fighter:removeDebuff(id)
	if self:hasDebuff(id) then
		self.buffs[id] = 0
		self.buffCount[id] = 0
		self:refresh()
	end
end

function Fighter:removeBuffOrDebuff(id)
	if self:hasBuff(id) or self:hasDebuff(id) then
		self.buffs[id] = 0
		self.buffCount[id] = 0
		self:refresh()
	end
end

function Fighter:removeStatusesOnDamage()
	for i = #self.statuses, 1, -1 do
		local id = self.statuses[i]
		if math.random() < dataStatus[id].removeOnDamage then
			self:removeStatus(id)
		end
	end
end

function Fighter:removeStatusesOnTurnEnd()
	for i = #self.statuses, 1, -1 do
		local id = self.statuses[i]
		if self.statusCount[id] > 0 then
			self.statusCount[id] = self.statusCount[id] - 1
			if self.statusCount[id] == 0 then
				self:removeStatus(id)
			end
		end
	end
end

function Fighter:removeStatusesOnBattleEnd()
	for i = #self.statuses, 1, -1 do
		local id = self.statuses[i]
		if dataStatus[id].removeOnBattleEnd then
			self:removeStatus(id)
		end
	end
end

function Fighter:removeBuffsOnTurnEnd()
	for i = 1, #self.buffs do
		if self.buffCount[i] > 0 then
			self.buffCount[i] = self.buffCount[i] - 1
			if self.buffCount[i] == 0 then
				self:removeBuffOrDebuff(i)
			end
		end
	end
end

function Fighter:removeBuffsOnBattleEnd()
	for i = 1, #self.buffs do
		self:removeBuffOrDebuff(i)
	end
end

function Fighter:clearStatuses()
	self.statuses = {}
	self.statusCount = {}
end

function Fighter:clearBuffs()
	self.buffs = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
	self.buffCount = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
end

function Fighter:clearAction()
	self:setAction(Action:new("None", 0, self))
end

function Fighter:setupAction()
	self:clearAction()
	if not self:isRestricted() then
		self:setupAutoAction()
	end
end

function Fighter:setupAutoAction()
end

function Fighter:refresh()
	self.hp = math.max(0, math.min(self:getMhp(), self.hp))
	self.mp = math.max(0, math.min(self:getMmp(), self.mp))
	if self.hp == 0 then
		self:addStatus(Fighter.STATUS_DEATH_ID)
	else
		self:removeStatus(Fighter.STATUS_DEATH_ID)
	end
end

return Fighter