local Object = require "core.Object"

local Group = Object:extend()

function Group:initialize()
	self.members = {}
end

function Group:isDefeated()
	for _, v in ipairs(self:getBattleMembers()) do
		if not v:isDead() then
			return false
		end
	end
	return true
end

function Group:isRestricted()
	for _, v in ipairs(self:getBattleMembers()) do
		if not v:isRestricted() then
			return false
		end
	end
	return true
end

function Group:getSize()
	return #self.members
end

function Group:getBattleSize()
	return math.min(#self.members, self:getMaxBattleSize())
end

function Group:getMaxBattleSize()
	return #self.members
end

function Group:getMember(index)
	return self.members[index]
end

function Group:getMemberIndex(member)
	for i, v in ipairs(self.members) do
		if v == member then
			return i
		end
	end
	return 0
end

function Group:getMembers()
	return self.members
end

function Group:getBattleMembers()
	return {unpack(self.members, 1, self:getBattleSize())}
end

function Group:addMember(member)
	table.insert(self.members, member)
end

function Group:removeMember(member)
	for i = #self.members, 1, -1 do
		if self.members[i] == member then
			table.remove(self.members, i)
		end
	end
end

function Group:clear()
	self.members = {}
end

function Group:setupBattlePosition()
end

return Group