local Object = require "core.Object"

local Indicator = Object:extend()

function Indicator:initialize()
	-- Image/quads
	self.image = RM:getSystemImage("indicator.png")
	self.quads = {}
	local qw = self:getQuadW()
	local qh = self:getQuadH()
	for i = 0, 3 do
		local qx = i * qw
		local qy = 0
		table.insert(self.quads, love.graphics.newQuad(qx, qy, qw, qh, qw * 4, qh))
	end
	-- State
	self.enabled = true
	self.index = 1
	-- Animation
	self.frame = 1
	self.timer = 0
end

function Indicator:getQuadW()
	return self.image:getWidth() / 4
end

function Indicator:getQuadH()
	return self.image:getHeight()
end

function Indicator:setEnabled(enabled)
	self.enabled = enabled
end

function Indicator:setIndex(index)
	self.index = index
end

function Indicator:update()
	if self.enabled then
		self.timer = self.timer + 1
		if self.timer > 128 / (2 ^ 4) then
			self.frame = self.frame % 4 + 1
			self.timer = 0
		end
	else
		self.frame = 1
		self.timer = 0
	end
end

function Indicator:draw()
	if not self.enabled then
		return
	end
	local a = gameActorGroup:getMember(self.index)
	local x = a:getX() - self:getQuadW() / 2
	local y = a:getY() - a:getH() - self:getQuadH()
	love.graphics.setShader()
	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(self.image, self.quads[self.frame], x, y)
end

return Indicator