local Object = require "core.Object"

local Transition = Object:extend()

function Transition:initialize()
	self.ts = 0
	self.to = 1
end

function Transition:isBusy()
	return (self.ts > 0 and self.to < 1) or (self.ts < 0 and self.to > 0)
end

function Transition:fadeIn(duration)
	if duration > 0 then
		self.ts = -1 / duration
		self.to = 1
	else
		self.ts = 0
		self.to = 0
	end
end

function Transition:fadeOut(duration)
	if duration > 0 then
		self.ts = 1 / duration
		self.to = 0
	else
		self.ts = 0
		self.to = 1
	end
end

function Transition:update()
	self.to = math.max(0, math.min(1, self.to + self.ts))
end

function Transition:draw()
	if self.to == 0 then
		return
	end
	love.graphics.setShader()
	love.graphics.setColor(0, 0, 0, self.to)
	love.graphics.rectangle("fill", 0, 0, VM:getW(), VM:getH())
end

return Transition