local VideoManager = require "manager.VideoManager"
local AudioManager = require "manager.AudioManager"
local InputManager = require "manager.InputManager"
local DataManager = require "manager.DataManager"
local ResourceManager = require "manager.ResourceManager"
local TextManager = require "manager.TextManager"
local StateManager = require "manager.StateManager"

local TitleState = require "state.TitleState"

local Transition = require "game.Transition"
local Display = require "game.Display"
local Actor = require "game.Actor"
local ActorGroup = require "game.ActorGroup"
local EnemyGroup = require "game.EnemyGroup"
local Battle = require "game.Battle"

---------------------------------------------------------------------------------------------------
-- love.load
---------------------------------------------------------------------------------------------------

function love.load()
	-- Random init
	math.randomseed(os.time())
	-- Love2D init
	love.keyboard.setKeyRepeat(true)
	love.graphics.setDefaultFilter("nearest")
	love.graphics.setBackgroundColor(0, 0, 0)
	-- Managers
	VM = VideoManager:new()
	AM = AudioManager:new()
	IM = InputManager:new()
	DM = DataManager:new()
	RM = ResourceManager:new()
	TM = TextManager:new()
	SM = StateManager:new()
	-- Data objects
	dataActors = DM:load("data/actors.json")
	dataEnemies = DM:load("data/enemies.json")
	dataGroups = DM:load("data/groups.json")
	dataStatus = DM:load("data/status.json")
	dataSkills = DM:load("data/skills.json")
	dataItems = DM:load("data/items.json")
	dataAnimations = DM:load("data/animations.json")
	-- Game objects
	gameTransition = Transition:new()
	gameDisplay = Display:new()
	gameActors = {}
	for i, _ in ipairs(dataActors) do
		table.insert(gameActors, Actor:new(i))
	end
	gameActorGroup = ActorGroup:new()
	gameEnemyGroup = EnemyGroup:new()
	gameBattle = Battle:new()
	-- State
	SM:pushState(TitleState:new())
end

---------------------------------------------------------------------------------------------------
-- love.update
---------------------------------------------------------------------------------------------------

function love.update(dt)
	SM:update()
	IM:update()
	AM:update()
end

---------------------------------------------------------------------------------------------------
-- love.draw
---------------------------------------------------------------------------------------------------

function love.draw()
	SM:draw()
end

---------------------------------------------------------------------------------------------------
-- Keyboard/focus callbacks
---------------------------------------------------------------------------------------------------

function love.keypressed(key)
	IM:keyPressed(key)
end

function love.keyreleased(key)
	IM:keyReleased(key)
end

function love.focus(focus)
	IM:focus(focus)
end