local Object = require "core.Object"

local AudioManager = Object:extend()

function AudioManager:initialize()
	self.data = {}
	self.currMusic = nil
	self.fs = 0
	self.fv = 0
end

function AudioManager:isBusy()
	return self.fs > 0 and self.fv > 0
end

function AudioManager:playSound(name)
	local path = "resources/sound/" .. name
	if not self.data[path] then
		self.data[path] = love.audio.newSource(path, "static")
	end
	local s = self.data[path]
	s:stop()
	s:play()
end

function AudioManager:playCursorSound()
	self:playSound("cursor.wav")
end

function AudioManager:playSelectSound()
	self:playSound("select.wav")
end

function AudioManager:playCancelSound()
	self:playSound("cancel.wav")
end

function AudioManager:playActionSound()
	self:playSound("action.wav")
end

function AudioManager:playEscapeSound()
	self:playSound("escape.wav")
end

function AudioManager:stopMusic()
	self.fs = 0
	self.fv = 0
	if self.currMusic then
		self.currMusic:stop()
	end
end

function AudioManager:fadeMusic(duration)
	if duration > 0 then
		self.fs = 1 / duration
		self.fv = 1
		if self.currMusic then
			self.currMusic:setVolume(self.fv)
		end
	else
		self.fs = 0
		self.fv = 0
		if self.currMusic then
			self.currMusic:stop()
		end
	end
end

function AudioManager:playMusic(name, loop)
	-- Load
	local path = "resources/music/" .. name
	if not self.data[path] then
		self.data[path] = love.audio.newSource(path, "stream")
	end
	-- Stop
	self.fs = 0
	self.fv = 0
	if self.currMusic and self.currMusic ~= self.data[path] then
		self.currMusic:stop()
	end
	-- Play
	self.currMusic = self.data[path]
	self.currMusic:setLooping(loop ~= nil and loop)
	self.currMusic:setVolume(1)
	self.currMusic:play()
end

function AudioManager:playTitleMusic()
	self:playMusic("bof4truthandfiction.ogg", true)
end

function AudioManager:playBattleMusic()
	self:playMusic("rnhbattle.ogg", true)
end

function AudioManager:playVictoryMusic()
	self:playMusic("ff3fnfar.ogg", true)
end

function AudioManager:update()
	if self.fs > 0 and self.fv > 0 then
		self.fv = math.max(0, self.fv - self.fs)
		if self.fv > 0 then
			if self.currMusic then
				self.currMusic:setVolume(self.fv)
			end
		else
			self:stopMusic()
		end
	end
end

return AudioManager