local Object = require "core.Object"

local InputManager = Object:extend()

InputManager.KEYMAP = {}
InputManager.KEYMAP["Up"] = {"up", "w"}
InputManager.KEYMAP["Right"] = {"right", "d"}
InputManager.KEYMAP["Down"] = {"down", "s"}
InputManager.KEYMAP["Left"] = {"left", "a"}
InputManager.KEYMAP["Select"] = {"z", "return", "space", "kpenter"}
InputManager.KEYMAP["Cancel"] = {"x", "escape", "insert", "kp0"}

function InputManager:initialize()
	self.keys = {}
end

function InputManager:isDown(key)
	for _, v in ipairs(InputManager.KEYMAP[key]) do
		if self.keys[v] then
			return true
		end
	end
	return false
end

function InputManager:isPressed(key)
	for _, v in ipairs(InputManager.KEYMAP[key]) do
		if self.keys[v] and self.keys[v][1] then
			return true
		end
	end
	return false
end

function InputManager:isRepeated(key)
	for _, v in ipairs(InputManager.KEYMAP[key]) do
		if self.keys[v] and self.keys[v][2] then
			return true
		end
	end
	return false
end

function InputManager:keyPressed(key)
	if not self.keys[key] then
		self.keys[key] = {
			true,
			true
		}
	else
		self.keys[key][2] = true
	end
end

function InputManager:keyReleased(key)
	self.keys[key] = nil
end

function InputManager:focus(focus)
	if not focus then
		for k, _ in pairs(self.keys) do
			self.keys[k] = nil
		end
	end
end

function InputManager:clear()
	for _, v in pairs(self.keys) do
		v[1] = false
		v[2] = false
	end
end

function InputManager:update()
	self:clear()
end

return InputManager