local Object = require "core.Object"

local StateManager = Object:extend()

function StateManager:initialize()
	self.stack = {}
	self.state = nil
	self.stateStopped = false
end

function StateManager:isStateChanging()
	return self.state ~= self.stack[#self.stack]
end

function StateManager:peekState()
	return self.stack[#self.stack]
end

function StateManager:pushState(state, clear)
	if clear then
		self.stack = {}
	end
	table.insert(self.stack, state)
end

function StateManager:prevState()
	table.remove(self.stack)
end

function StateManager:update()
	if self.state ~= self:peekState() then
		if self.state and not self.stateStopped then
			self.stateStopped = true
			self.state:stop()
		end
		if not self.state or not self.state:isBusy() then
			self.stateStopped = false
			self.state = self:peekState()
			if self.state then
				self.state:start()
			else
				love.event.quit()
			end
		end
	end
	if self.state then
		self.state:update()
	end
end

function StateManager:draw()
	if self.state then
		self.state:draw()
	end
end

return StateManager