local DamageUtil = {}

DamageUtil.CRITICAL_RATE = 1.75

function DamageUtil:evalFormula(a, b, formula)
    -- User/target (a/b)
    local env = {
        a = setmetatable({}, {
            __index = function(t, k)
                return a["get" .. k:sub(1, 1):upper() .. k:sub(2)](a)
            end
        }),
        b = setmetatable({}, {
            __index = function(t, k)
                return b["get" .. k:sub(1, 1):upper() .. k:sub(2)](b)
            end
        })
    }
    -- Math
    setmetatable(env, {
        __index = math
    })
    return setfenv(loadstring("return " .. formula), env)()
end

function DamageUtil:calcValue(a, b, formula, variance, type, critical, attributes)
    local value = math.max(0, self:evalFormula(a, b, formula))
    -- Variance
    value = math.max(0, value - value * variance / 2 + value * variance * math.random())
    -- Sign
    value = value * ((type == 1 and -1) or (type == 2 and 1) or 0) * ((b:isDamageInverted() and -1) or 1)
    -- Damage rate
    value = value * ((type == 1 and b:getDamageRate()) or 1)
    -- Critical
    value = value * ((critical and DamageUtil.CRITICAL_RATE) or 1)
    -- Attributes
    for _, v in ipairs(attributes) do
        value = value * b:getAttributeRate(v)
    end
    -- Round
    value = math.floor(value + 0.5)
    return value
end

function DamageUtil:calcDrain(a, value, rate)
    return math.floor(value * rate * -1 * ((a:isDamageInverted() and -1) or 1) + 0.5)
end

function DamageUtil:calcHpRecovery(a)
    return math.floor(a:getMhp() * a:getHpRecoveryRate() * ((a:isDamageInverted() and -1) or 1) + 0.5)
end

function DamageUtil:calcMpRecovery(a)
    return math.floor(a:getMmp() * a:getMpRecoveryRate() * ((a:isDamageInverted() and -1) or 1) + 0.5)
end

return DamageUtil