local utf8 = require "utf8"
local Object = require "core.Object"

local Window = Object:extend()

Window.COLOR_BG = 0
Window.COLOR_FG = 1
Window.COLOR_FG_DISABLED = 2
Window.COLOR_FG_BLUE = 3
Window.COLOR_FG_GREEN = 4
Window.COLOR_FG_YELLOW = 5
Window.COLOR_FG_ORANGE = 6
Window.COLOR_FG_RED = 7
Window.COLOR_FG_PURPLE = 8
Window.COLOR_FG_VIOLET = 9
Window.COLOR_FG_DARK_BLUE = 10
Window.COLOR_FG_DARK_GREEN = 11
Window.COLOR_FG_DARK_YELLOW = 12
Window.COLOR_FG_DARK_ORANGE = 13
Window.COLOR_FG_DARK_RED = 14
Window.COLOR_FG_DARK_PURPLE = 15
Window.COLOR_FG_DARK_VIOLET = 16

Window.ALIGN_LEFT = 1
Window.ALIGN_CENTER = 2
Window.ALIGN_RIGHT = 3

function Window:initialize(x, y, w, h)
    -- Font
    self.font = RM:getFont("default.ttf", 32)
    -- Skin image/quads
    self.skinImage = RM:getSystemImage("skin.png")
    self.skinQuads = {}
    -- BG (1)
    table.insert(self.skinQuads, love.graphics.newQuad(0, 0, 64, 64, 128, 128))
    -- FG (2-9)
    table.insert(self.skinQuads, love.graphics.newQuad(64, 0, 16, 16, 128, 128))
    table.insert(self.skinQuads, love.graphics.newQuad(80, 0, 32, 16, 128, 128))
    table.insert(self.skinQuads, love.graphics.newQuad(112, 0, 16, 16, 128, 128))
    table.insert(self.skinQuads, love.graphics.newQuad(64, 16, 16, 32, 128, 128))
    table.insert(self.skinQuads, love.graphics.newQuad(112, 16, 16, 32, 128, 128))
    table.insert(self.skinQuads, love.graphics.newQuad(64, 48, 16, 16, 128, 128))
    table.insert(self.skinQuads, love.graphics.newQuad(80, 48, 32, 16, 128, 128))
    table.insert(self.skinQuads, love.graphics.newQuad(112, 48, 16, 16, 128, 128))
    -- Arrows (10: top, 11: bottom)
    table.insert(self.skinQuads, love.graphics.newQuad(80, 16, 32, 16, 128, 128))
    table.insert(self.skinQuads, love.graphics.newQuad(80, 32, 32, 16, 128, 128))
    -- Cursor (12: enabled, 13: disabled)
    table.insert(self.skinQuads, love.graphics.newQuad(0, 64, 64, 64, 128, 128))
    table.insert(self.skinQuads, love.graphics.newQuad(64, 64, 64, 64, 128, 128))
    -- Face/battleface image/quads
    self.faceImage = RM:getSystemImage("face.png")
    self.battlefaceImage = RM:getSystemImage("battleface.png")
    self.faceQuads = {}
	self.battlefaceQuads = {}
	local fqw = self:getFaceQuadW()
	local fqh = self:getFaceQuadH()
	local fiw = fqw * 4
	local fih = fqh * 4
	local bqw = self:getBattlefaceQuadW()
	local bqh = self:getBattlefaceQuadH()
	local biw = bqw * 4
	local bih = bqh * 4
    for y = 0, 3 do
		for x = 0, 3 do
            table.insert(self.faceQuads, love.graphics.newQuad(x * fqw, y * fqh, fqw, fqh, fiw, fih))
			table.insert(self.battlefaceQuads, love.graphics.newQuad(x * bqw, y * bqh, bqw, bqh, biw, bih))
        end
    end
    -- Callbacks
	self.callbacks = {}
	-- State
	self.visible = true
	self.enabled = true
    self.opening = false
    self.closing = false
	self.x = x
    self.y = y
	self.w = w
    self.h = h
	self.ox = 0
	self.oy = 0
	self.os = 0
	self.openness = 0
	-- Blink
	self.blink = false
	self.blinkTimer = 0
end

function Window:isVisible()
    return self.visible
end

function Window:isEnabled()
    return self.enabled
end

function Window:isOpening()
    return self.opening
end

function Window:isClosing()
    return self.closing
end

function Window:isOpen()
    return self.openness == 1
end

function Window:isClosed()
    return self.openness == 0
end

function Window:hasCallback(name)
    return self.callbacks[name]
end

function Window:getHPad()
	return 24
end

function Window:getVPad()
	return 16
end

function Window:getLineH()
	return 32
end

function Window:getDefaultSpacing()
	return 4
end

function Window:getContentW()
	return self.w - self:getHPad() * 2
end

function Window:getContentH()
	return self.h - self:getVPad() * 2
end

function Window:getFontAscent()
	return self.font:getAscent()
end

function Window:getFontDescent()
	return self.font:getDescent()
end

function Window:getFaceQuadW()
	return 96
end

function Window:getFaceQuadH()
	return 96
end

function Window:getBattlefaceQuadW()
	return 96
end

function Window:getBattlefaceQuadH()
	return 32
end

function Window:getSkinColor(color)
    -- BG
	if color == 0 then
		return 0, 0, 0, 0.5
	end
	-- FG
	if color == 1 then
		return 1, 1, 1
	end
	-- FG (disabled)
	if color == 2 then
		return 1, 1, 1, 0.5
	end
	-- FG (blue)
	if color == 3 then
		return 0.5, 0.65, 0.95
	end
	-- FG (green)
	if color == 4 then
		return 0.5, 1, 0.5
	end
	-- FG (yellow)
	if color == 5 then
		return 1, 1, 0.4
	end
	-- FG (orange)
	if color == 6 then
		return 1, 0.6, 0.4
	end
	-- FG (red)
	if color == 7 then
		return 1, 0.4, 0.4
	end
	-- FG (purple)
	if color == 8 then
		return 0.9, 0.5, 1
	end
	-- FG (violet)
	if color == 9 then
		return 0.7, 0.4, 1
	end
	-- FG (dark blue)
	if color == 10 then
		return 0.05, 0.2, 0.45
	end
	-- FG (dark green)
	if color == 11 then
		return 0, 0.3, 0
	end
	-- FG (dark yellow)
	if color == 12 then
		return 0.3, 0.3, 0
	end
	-- FG (dark orange)
	if color == 13 then
		return 0.3, 0.15, 0
	end
	-- FG (dark red)
	if color == 14 then
		return 0.3, 0, 0
	end
	-- FG (dark purple)
	if color == 15 then
		return 0.2, 0.05, 0.15
	end
	-- FG (dark violet)
	if color == 16 then
		return 0.2, 0.05, 0.3
	end
	return 1, 1, 1
end

function Window:getTextW(text)
	return self.font:getWidth(text) + 2
end

function Window:setCallback(name, callback)
    self.callbacks[name] = callback
end

function Window:setVisible(visible)
	self.visible = visible
end

function Window:setEnabled(enabled)
	self.enabled = enabled
end

function Window:playCursorSound()
	AM:playCursorSound()
end

function Window:playSelectSound()
	AM:playSelectSound()
end

function Window:playCancelSound()
	AM:playCancelSound()
end

function Window:open(duration)
    if (duration or 4) > 0 then
        self.os = 1 / (duration or 4)
    else
        self.openness = 1
        self.os = 0
    end
    self.opening = not self:isOpen()
    self.closing = false
end

function Window:close(duration)
    if (duration or 4) > 0 then
        self.os = 1 / (duration or 4)
    else
        self.openness = 0
        self.os = 0
    end
    self.opening = false
    self.closing = not self:isClosed()
end

function Window:invokeCallback(name)
	self.callbacks[name]()
end

function Window:update()
    if self.visible and self.enabled and self:isOpen() then
		self.blinkTimer = self.blinkTimer + 1
		if self.blinkTimer >= 32 then
			self.blink = not self.blink
			self.blinkTimer = 0
		end
	else
		self.blink = false
		self.blinkTimer = 0
	end
    if self.opening then
        self.openness = math.min(1, self.openness + self.os)
		if self:isOpen() then
            self.opening = false
			if self:hasCallback("Open") then
				self:invokeCallback("Open")
            end
        end
    end
    if self.closing then
        self.openness = math.max(0, self.openness - self.os)
        if self:isClosed() then
            self.closing = false
			if self:hasCallback("Closed") then
				self:invokeCallback("Closed")
            end
        end
    end
end

function Window:drawText(text, x, y, w, h, color, align)
	local tw = self:getTextW(text)
	local rw = w or tw
	local rh = h or self:getLineH()
	if tw > rw then
		while utf8.len(text) > 0 and self:getTextW(text .. "...") > rw do
			text = string.sub(text, 1, utf8.offset(text, -1) - 1)
		end
		text = text .. "..."
		tw = self:getTextW(text)
	end
	love.graphics.setShader()
	love.graphics.setFont(self.font)
	love.graphics.push()
	love.graphics.translate(x, y + (rh + self:getFontDescent() - self:getFontAscent()) / 2)
	if align == Window.ALIGN_CENTER then
		love.graphics.translate((rw - math.min(rw, tw)) / 2, 0)
	end
	if align == Window.ALIGN_RIGHT then
		love.graphics.translate(rw - math.min(rw, tw), 0)
	end
	love.graphics.setColor(self:getSkinColor(Window.COLOR_BG))
	love.graphics.print(text, 2, 2)
	love.graphics.setColor(self:getSkinColor(color or Window.COLOR_FG))
	love.graphics.print(text, 0, 0)
	love.graphics.pop()
end

function Window:drawBar(value1, value2, x, y, w, h, bgColor, fgColor)
	local bw1 = w - 2
	local bw2 = bw1 * value1 / value2
	local bh = 2
	local bx = x
	local by = y + h - bh - 2
	love.graphics.setShader()
	-- BG
	love.graphics.setColor(self:getSkinColor(Window.COLOR_BG))
	love.graphics.rectangle("fill", bx + 2, by + 2, bw1, bh)
	-- BG (bar)
	love.graphics.setColor(self:getSkinColor(bgColor))
	love.graphics.rectangle("fill", bx, by, bw1, bh)
	-- FG (bar)
	love.graphics.setColor(self:getSkinColor(fgColor))
	love.graphics.rectangle("fill", bx, by, bw2, bh)
end

function Window:drawActorStat(text, value1, value2, x, y, w, h, textColor, valueColor, barBgColor, barFgColor)
	self:drawBar(value1, value2, x, y, w, h, barBgColor, barFgColor)
	self:drawText(text, x, y, w, h, textColor)
	self:drawText(string.format("%3d/%3d", value1, value2), x, y, w, h, valueColor, Window.ALIGN_RIGHT)
end

function Window:drawActorFace(id, x, y)
	local a = gameActors[id]
	love.graphics.setShader()
	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(self.faceImage, self.faceQuads[a:getFace()], x, y)
end

function Window:drawActorBattleface(id, x, y)
	local a = gameActors[id]
	love.graphics.setShader()
	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(self.battlefaceImage, self.battlefaceQuads[a:getFace()], x, y)
end

function Window:drawActorName(id, x, y, w, h)
	local a = gameActors[id]
	self:drawText(a:getName(), x, y, w, h, (a:isDead() and Window.COLOR_FG_RED) or (a:isHpCritical() and Window.COLOR_FG_YELLOW) or Window.COLOR_FG)
end

function Window:drawActorHp(id, x, y, w, h)
	local a = gameActors[id]
	self:drawActorStat(TM.HP, a:getHp(), a:getMhp(), x, y, w, h, Window.COLOR_FG_BLUE, (a:isDead() and Window.COLOR_FG_RED) or (a:isHpCritical() and Window.COLOR_FG_YELLOW) or Window.COLOR_FG, Window.COLOR_FG_DARK_VIOLET, Window.COLOR_FG_VIOLET)
end

function Window:drawActorMp(id, x, y, w, h)
	local a = gameActors[id]
	self:drawActorStat(TM.MP, a:getMp(), a:getMmp(), x, y, w, h, Window.COLOR_FG_BLUE, (a:isMpCritical() and Window.COLOR_FG_YELLOW) or Window.COLOR_FG, Window.COLOR_FG_DARK_PURPLE, Window.COLOR_FG_PURPLE)
end

function Window:drawSkin()
	love.graphics.setShader()
	love.graphics.setColor(1, 1, 1)
	-- BG
	love.graphics.draw(self.skinImage, self.skinQuads[1], 0, 0, 0, self.w / 64, self.h / 64)
	-- FG
	love.graphics.draw(self.skinImage, self.skinQuads[2], 0, 0)
	love.graphics.draw(self.skinImage, self.skinQuads[3], 16, 0, 0, (self.w - 32) / 32, 1)
	love.graphics.draw(self.skinImage, self.skinQuads[4], self.w - 16, 0)
	love.graphics.draw(self.skinImage, self.skinQuads[5], 0, 16, 0, 1, (self.h - 32) / 32)
	love.graphics.draw(self.skinImage, self.skinQuads[6], self.w - 16, 16, 0, 1, (self.h - 32) / 32)
	love.graphics.draw(self.skinImage, self.skinQuads[7], 0, self.h - 16)
	love.graphics.draw(self.skinImage, self.skinQuads[8], 16, self.h - 16, 0, (self.w - 32) / 32, 1)
	love.graphics.draw(self.skinImage, self.skinQuads[9], self.w - 16, self.h - 16)
end

function Window:drawTopArrow()
	if not self.enabled or self.blink then
		return
	end
	love.graphics.setShader()
	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(self.skinImage, self.skinQuads[10], self.w / 2 - 16, 0)
end

function Window:drawBottomArrow()
	if not self.enabled or self.blink then
		return
	end
	love.graphics.setShader()
	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(self.skinImage, self.skinQuads[11], self.w / 2 - 16, self.h - 16)
end

function Window:drawCursor(x, y, w, h, enabled)
	love.graphics.setShader()
	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(self.skinImage, (enabled and self.skinQuads[12]) or self.skinQuads[13], x, y, 0, w / 64, h / 64)
end

function Window:drawInterface()
end

function Window:drawContent()
end

function Window:draw()
	if not self.visible then
		return
	end
	-- Skin
	if not self:isClosed() then
		local tx = self.x
		local ty = self.y + self.h / 2
		love.graphics.push()
		love.graphics.translate(tx, ty)
		love.graphics.scale(1, self.openness)
		love.graphics.translate(0, -self.h / 2)
		self:drawSkin()
		love.graphics.pop()
	end
	-- Interface/content
	if self:isOpen() then
		local sw = self:getContentW()
		local sh = self:getContentH()
		local sx = self.x + self:getHPad()
		local sy = self.y + self:getVPad()
		local tx = self.x
		local ty = self.y
		local cx = self:getHPad() - self.ox
		local cy = self:getVPad() - self.oy
		love.graphics.push()
		love.graphics.translate(tx, ty)
		self:drawInterface()
		love.graphics.push()
		love.graphics.translate(cx, cy)
		love.graphics.setScissor(sx, sy, sw, sh)
		self:drawContent()
		love.graphics.setScissor()
		love.graphics.pop()
		love.graphics.pop()
	end
end

return Window